@@ -1,15 +1,17 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
+use std::collections::HashMap;
+
 use gst::prelude::DeviceExt;
 use gtk::glib;
 use gtk::prelude::*;
 use gtk::subclass::prelude::*;
-use std::collections::HashMap;
 
 mod imp {
-    use super::*;
+    use std::cell::OnceCell;
 
     use glib::Properties;
-    use once_cell::unsync::OnceCell;
+
+    use super::*;
 
     #[derive(Debug, Default, Properties)]
     #[properties(wrapper_type = super::Camera)]
@@ -24,19 +26,8 @@ mod imp {
         type Type = super::Camera;
     }
 
-    impl ObjectImpl for Camera {
-        fn properties() -> &'static [glib::ParamSpec] {
-            Self::derived_properties()
-        }
-
-        fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-            Self::derived_property(self, id, pspec)
-        }
-
-        fn set_property(&self, id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
-            Self::derived_set_property(self, id, value, pspec)
-        }
-    }
+    #[glib::derived_properties]
+    impl ObjectImpl for Camera {}
 }
 
 glib::wrapper! {
@@ -87,8 +78,8 @@ impl Camera {
     ///
     /// # Returns
     ///
-    /// a [`HashMap`][std::collections::HashMap], with the property name as the key
-    /// and a [`GValue`][gtk::glib::Value] as the value.
+    /// a [`HashMap`][std::collections::HashMap], with the property name as the
+    /// key and a [`GValue`][gtk::glib::Value] as the value.
     pub fn properties(&self) -> HashMap<&'static str, glib::SendValue> {
         self.device()
             .properties()
@@ -165,7 +156,7 @@ impl Camera {
 fn create_element(device: &gst::Device) -> Option<(gst::Element, gst::Element)> {
     use gst::prelude::*;
 
-    let bin = gst::Bin::new(None);
+    let bin = gst::Bin::new();
 
     let device_src = device.create_element(None).ok()?;
     device_src.set_property("client-name", crate::APP_ID.get().unwrap());
@@ -192,20 +183,19 @@ fn create_element(device: &gst::Device) -> Option<(gst::Element, gst::Element)>
         .build()
         .unwrap();
 
-    bin.add_many(&[&device_src, &capsfilter, &decodebin3, &videoflip])
+    bin.add_many([&device_src, &capsfilter, &decodebin3, &videoflip])
         .unwrap();
-    gst::Element::link_many(&[&device_src, &capsfilter, &decodebin3]).unwrap();
+    gst::Element::link_many([&device_src, &capsfilter, &decodebin3]).unwrap();
 
     decodebin3.connect_pad_added(glib::clone!(@weak videoflip => move |_, pad| {
-        // TODO Use is_some_and once stabilized
-        if pad.stream().map(|stream| matches!(stream.stream_type(), gst::StreamType::VIDEO)).unwrap_or_default() {
+        if pad.stream().is_some_and(|stream| matches!(stream.stream_type(), gst::StreamType::VIDEO)) {
             pad.link(&videoflip.static_pad("sink").unwrap())
                .expect("Failed to link decodebin3:video_%u pad with videoflip:sink");
         }
     }));
 
     let pad = videoflip.static_pad("src").unwrap();
-    let ghost_pad = gst::GhostPad::with_target(Some("src"), &pad).unwrap();
+    let ghost_pad = gst::GhostPad::with_target(&pad).unwrap();
     ghost_pad.set_active(true).unwrap();
 
     bin.add_pad(&ghost_pad).unwrap();
