@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 use std::cell::RefCell;
+use std::os::fd::{FromRawFd, OwnedFd};
 use std::os::unix::io::RawFd;
 use std::sync::Once;
 
@@ -11,17 +12,19 @@ use gtk::{gio, glib};
 static STARTED: Once = Once::new();
 
 mod imp {
-    use super::*;
+    use std::cell::OnceCell;
 
     use once_cell::sync::Lazy;
-    use once_cell::unsync::OnceCell;
+
+    use super::*;
 
     #[derive(Debug, Default)]
     pub struct DeviceProvider {
         pub inner: OnceCell<gst::DeviceProvider>,
         pub cameras: RefCell<Vec<crate::Camera>>,
+        pub bus_watch: OnceCell<gst::bus::BusWatchGuard>,
 
-        pub fd: RefCell<Option<RawFd>>,
+        pub fd: RefCell<Option<OwnedFd>>,
     }
 
     impl DeviceProvider {
@@ -38,8 +41,15 @@ mod imp {
 
         pub fn remove(&self, device: crate::Camera) {
             let guard = self.cameras.borrow();
-            let Some((pos, _)) =  guard.iter().enumerate().find(|(_idx, x)| x.target_object() == device.target_object()) else {
-                log::error!("Tried to remove camera with target-object {:?} but it wasn't in the vec?", device.target_object());
+            let Some((pos, _)) = guard
+                .iter()
+                .enumerate()
+                .find(|(_idx, x)| x.target_object() == device.target_object())
+            else {
+                log::error!(
+                    "Tried to remove camera with target-object {:?} but it wasn't in the vec?",
+                    device.target_object()
+                );
                 return;
             };
             drop(guard);
@@ -67,7 +77,8 @@ mod imp {
             self.cameras
                 .borrow()
                 .get(position as usize)
-                .map(|o| o.clone().upcast::<glib::Object>())
+                .cloned()
+                .and_upcast()
         }
     }
 
@@ -86,15 +97,7 @@ mod imp {
             if inner.is_started() {
                 inner.stop();
             }
-            let bus = inner.bus();
-            let _ = bus.remove_watch();
-            if let Some(raw_fd) = self.fd.take() {
-                unsafe {
-                    // FIXME Replace with a OwnedFd once
-                    // https://github.com/bilelmoussaoui/ashpd/pull/104 is merged.
-                    libc::close(raw_fd);
-                }
-            }
+            inner.set_property("fd", -1);
         }
 
         fn signals() -> &'static [glib::subclass::Signal] {
@@ -143,8 +146,9 @@ impl DeviceProvider {
         use once_cell::sync::Lazy;
 
         struct Wrapper(ThreadGuard<crate::DeviceProvider>);
-        // SAFETY: We only ever hand out a reference to the contained object on the one thread
-        // it was created one so no two threads can access it at the same time.
+        // SAFETY: We only ever hand out a reference to the contained object on the one
+        // thread it was created one so no two threads can access it at the same
+        // time.
         unsafe impl Sync for Wrapper {}
 
         static SINGLETON: Lazy<Wrapper> = Lazy::new(|| {
@@ -169,13 +173,14 @@ impl DeviceProvider {
 
         STARTED.call_once(glib::clone!(@weak self as obj, @weak provider => move || {
             let bus = provider.bus();
-            bus.add_watch_local(
-                glib::clone!(@weak obj => @default-return glib::Continue(false),
+            let watch = bus.add_watch_local(
+                glib::clone!(@weak obj => @default-return glib::ControlFlow::Break,
                     move |_, msg| {
                         obj.handle_message(msg);
-                        glib::Continue(true)
+                        glib::ControlFlow::Continue
                     })
             ).expect("Failed to add bus watch");
+            obj.imp().bus_watch.set(watch).unwrap();
         }));
 
         Ok(())
@@ -194,8 +199,9 @@ impl DeviceProvider {
 
     /// Set a valid file description to load and monitor cameras from.
     ///
-    /// This file descriptor should point to a valid Pipewire remote where camera nodes are available.
-    /// This provider should only provide camera nodes.
+    /// This file descriptor should point to a valid Pipewire remote where
+    /// camera nodes are available. This provider should only provide camera
+    /// nodes.
     ///
     /// One way to get a valid descriptor is with the [`org.freedesktop.portal.Camera`](https://flatpak.github.io/xdg-desktop-portal/#gdbus-org.freedesktop.portal.Camera)
     /// XDG portal, using the `OpenPipeWireRemote()` method.
@@ -206,8 +212,9 @@ impl DeviceProvider {
         let provider = self.imp().inner.get().unwrap();
         log::debug!("Starting device provider with file descriptor: {fd}");
         if provider.has_property("fd", Some(RawFd::static_type())) {
-            provider.set_property("fd", &fd);
-            self.imp().fd.replace(Some(fd));
+            provider.set_property("fd", fd);
+            let owned_fd = unsafe { OwnedFd::from_raw_fd(fd) };
+            self.imp().fd.replace(Some(owned_fd));
             Ok(())
         } else {
             log::warn!("Pipewire device provider does not have the `fd` property, please update to a version newer than 0.3.64");
