@@ -6,17 +6,18 @@
 //!
 //! See also
 //!
-//! - [Snapshot](https://gitlab.gnome.org/Incubator/snapshot)
+//! - [Snapshot](https://gitlab.gnome.org/GNOME/snapshot)
 //!
 //! # Usage
 //!
 //! Aperture needs to initialized before use.
-//! This can be done by calling [`fn@init`] on [`startup`](fn@gtk::gio::prelude::ApplicationExt::connect_startup).
+//! This can be done by calling [`fn@init`] on
+//! [`startup`](fn@gtk::gio::prelude::ApplicationExt::connect_startup).
+
+use std::sync::{Once, OnceLock};
 
 use gst::prelude::*;
 use gtk::glib;
-use once_cell::sync::OnceCell;
-use std::sync::Once;
 
 mod camera;
 mod device_provider;
@@ -29,22 +30,23 @@ pub use camera::Camera;
 pub use device_provider::DeviceProvider;
 pub use enums::{CameraLocation, CodeType, ViewfinderState};
 pub use error::{CaptureError, PipewireError};
+pub(crate) use pipeline_tee::PipelineTee;
 pub use viewfinder::Viewfinder;
 
-pub(crate) use pipeline_tee::PipelineTee;
-
-pub(crate) static APP_ID: OnceCell<&'static str> = OnceCell::new();
+pub(crate) static APP_ID: OnceLock<&'static str> = OnceLock::new();
 
 static IS_INIT: Once = Once::new();
-static VERSION: &'static str = env!("CARGO_PKG_VERSION");
+const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 /// Initializes the library
 ///
 /// This function can be used instead of [`fn@gtk::init`] and [`fn@gst::init`]
 /// as it initializes GTK and GStreamer implicitly.
 ///
-/// This function must be called on the [`startup`](fn@gtk::gio::prelude::ApplicationExt::connect_startup)
-/// of the [`GApplication`][`gtk::gio::Application`]. This function is idempotent.
+/// This function must be called on the
+/// [`startup`](fn@gtk::gio::prelude::ApplicationExt::connect_startup)
+/// of the [`GApplication`][`gtk::gio::Application`]. This function is
+/// idempotent.
 pub fn init(app_id: &'static str) {
     IS_INIT.call_once(|| {
         APP_ID.set(app_id).unwrap();
@@ -80,7 +82,7 @@ pub fn init(app_id: &'static str) {
 ///
 /// The Aperture version
 pub fn version() -> &'static str {
-    &*VERSION
+    VERSION
 }
 
 /// Use this function to check if Aperture has been initialized with
