@@ -1,12 +1,12 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
+use std::path::Path;
+use std::path::PathBuf;
+
 use gst::prelude::*;
 use gtk::prelude::*;
 use gtk::subclass::prelude::*;
 use gtk::{gdk, gio, glib, graphene};
-
 use once_cell::sync::Lazy;
-use std::path::Path;
-use std::path::PathBuf;
 
 use crate::ViewfinderState;
 
@@ -14,12 +14,13 @@ const BARCODE_TIMEOUT: u32 = 1;
 const PROVIDER_TIMEOUT: u64 = 2;
 
 mod imp {
-    use super::*;
+    use std::cell::Cell;
+    use std::cell::OnceCell;
+    use std::cell::RefCell;
 
     use glib::Properties;
-    use once_cell::unsync::OnceCell;
-    use std::cell::Cell;
-    use std::cell::RefCell;
+
+    use super::*;
 
     #[derive(Debug, Default, Properties)]
     #[properties(wrapper_type = super::Viewfinder)]
@@ -37,6 +38,7 @@ mod imp {
         pub camerabin: OnceCell<gst::Element>,
         pub sink_paintable: OnceCell<gst::Element>,
         pub tee: OnceCell<crate::PipelineTee>,
+        pub bus_watch: OnceCell<gst::bus::BusWatchGuard>,
 
         // TODO Port to gio::Task,
         pub is_recording_video: RefCell<Option<PathBuf>>,
@@ -170,19 +172,8 @@ mod imp {
         }
     }
 
+    #[glib::derived_properties]
     impl ObjectImpl for Viewfinder {
-        fn properties() -> &'static [glib::ParamSpec] {
-            Self::derived_properties()
-        }
-
-        fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-            Self::derived_property(self, id, pspec)
-        }
-
-        fn set_property(&self, id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
-            Self::derived_set_property(self, id, value, pspec)
-        }
-
         fn constructed(&self) {
             self.parent_constructed();
 
@@ -194,14 +185,14 @@ mod imp {
             self.camerabin.set(camerabin.clone()).unwrap();
 
             let bus = self.camerabin().bus().unwrap();
-            bus.add_watch_local(
-                glib::clone!(@weak obj => @default-return glib::Continue(false), move |_, msg| {
+            let watch = bus.add_watch_local(
+                glib::clone!(@weak obj => @default-return glib::ControlFlow::Break, move |_, msg| {
                     obj.on_bus_message(msg);
-
-                    glib::Continue(true)
+                    glib::ControlFlow::Continue
                 }),
             )
             .unwrap();
+            self.bus_watch.set(watch).unwrap();
 
             let tee = crate::PipelineTee::new();
 
@@ -227,8 +218,7 @@ mod imp {
                 convert.link(&paintablesink).unwrap();
 
                 bin.add_pad(
-                    &gst::GhostPad::with_target(Some("sink"), &convert.static_pad("sink").unwrap())
-                        .unwrap(),
+                    &gst::GhostPad::with_target(&convert.static_pad("sink").unwrap()).unwrap(),
                 )
                 .unwrap();
 
@@ -240,6 +230,8 @@ mod imp {
 
             self.sink_paintable.set(paintablesink).unwrap();
 
+            self.picture
+                .set_accessible_role(gtk::AccessibleRole::Presentation);
             self.picture.set_hexpand(true);
             self.picture.set_vexpand(true);
             self.picture.set_content_fit(gtk::ContentFit::Fill);
@@ -292,9 +284,6 @@ mod imp {
         }
 
         fn dispose(&self) {
-            if let Some(bus) = self.camerabin().bus() {
-                let _ = bus.remove_watch();
-            };
             if self.is_recording_video.borrow().is_some() {
                 if let Err(err) = self.obj().stop_recording() {
                     log::error!("Could not stop recording: {err}");
@@ -451,27 +440,32 @@ impl Viewfinder {
     ///
     /// # Returns
     ///
-    /// an aspect ratio calculated with width/height, or 0 for no valid aspect ratio.
+    /// an aspect ratio calculated with width/height, or 0 for no valid aspect
+    /// ratio.
     pub fn aspect_ratio(&self) -> f64 {
         let imp = self.imp();
         if let Some(paintable) = imp.picture.paintable() {
-            return paintable.intrinsic_aspect_ratio();
+            paintable.intrinsic_aspect_ratio()
         } else {
-            return 0.0;
+            0.0
         }
     }
 
     /// Takes a picture.
     ///
-    /// The recording will be saved to `location`. This method throws an error if:
+    /// The recording will be saved to `location`. This method throws an error
+    /// if:
     ///  - we are already recording or taking a picture
-    ///  - the [`fn@Viewfinder::state`] of the camera is not [`ViewfinderState::Ready`][crate::ViewfinderState::Ready].
+    ///  - the [`fn@Viewfinder::state`] of the camera is not
+    ///    [`ViewfinderState::Ready`][crate::ViewfinderState::Ready].
     ///
-    /// This operation may take a while. The resolution might be changed temporarily,
-    /// autofocusing might take place, etc. Basically everything you'd expect
-    /// to happen when you click the photo button in a camera app.
+    /// This operation may take a while. The resolution might be changed
+    /// temporarily, autofocusing might take place, etc. Basically
+    /// everything you'd expect to happen when you click the photo button in
+    /// a camera app.
     ///
-    /// The [`picture-done`](#picture-done) signal will be emitted when this operation ends.
+    /// The [`picture-done`](#picture-done) signal will be emitted when this
+    /// operation ends.
     pub fn take_picture<P: AsRef<Path>>(&self, location: P) -> Result<(), crate::CaptureError> {
         let imp = self.imp();
 
@@ -502,9 +496,11 @@ impl Viewfinder {
 
     /// Starts recording a video.
     ///
-    /// The recording will be saved to `location`. This method throws an error if:
+    /// The recording will be saved to `location`. This method throws an error
+    /// if:
     ///  - we are already recording or taking a picture
-    ///  - the [`fn@Viewfinder::state`] of the camera is not [`ViewfinderState::Ready`][crate::ViewfinderState::Ready].
+    ///  - the [`fn@Viewfinder::state`] of the camera is not
+    ///    [`ViewfinderState::Ready`][crate::ViewfinderState::Ready].
     pub fn start_recording<P: AsRef<Path>>(&self, location: P) -> Result<(), crate::CaptureError> {
         let imp = self.imp();
 
@@ -539,9 +535,11 @@ impl Viewfinder {
     ///
     /// This method throws an error if:
     /// - [`fn@Viewfinder::start_recording`] hasn't been called
-    /// - There is another [`fn@Viewfinder::stop_recording`] operation in progress.
+    /// - There is another [`fn@Viewfinder::stop_recording`] operation in
+    ///   progress.
     ///
-    /// The [`recording-done`](#recording-done) signal will be emitted when this operation ends.
+    /// The [`recording-done`](#recording-done) signal will be emitted when this
+    /// operation ends.
     pub fn stop_recording(&self) -> Result<(), crate::CaptureError> {
         let imp = self.imp();
 
@@ -764,17 +762,17 @@ impl Viewfinder {
 }
 
 fn create_zbar_bin() -> Result<gst::Element, glib::BoolError> {
-    let bin = gst::Bin::new(None);
+    let bin = gst::Bin::new();
 
     let videoconvert = gst::ElementFactory::make("videoconvert").build()?;
     let zbar = gst::ElementFactory::make("zbar").build()?;
     let fakesink = gst::ElementFactory::make("fakesink").build()?;
 
-    bin.add_many(&[&videoconvert, &zbar, &fakesink]).unwrap();
-    gst::Element::link_many(&[&videoconvert, &zbar, &fakesink]).unwrap();
+    bin.add_many([&videoconvert, &zbar, &fakesink]).unwrap();
+    gst::Element::link_many([&videoconvert, &zbar, &fakesink]).unwrap();
 
     let pad = videoconvert.static_pad("sink").unwrap();
-    let ghost_pad = gst::GhostPad::with_target(Some("sink"), &pad).unwrap();
+    let ghost_pad = gst::GhostPad::with_target(&pad).unwrap();
     ghost_pad.set_active(true).unwrap();
     bin.add_pad(&ghost_pad).unwrap();
 
