@@ -1,42 +1,44 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-use ashpd::desktop::camera;
-use gettextrs::gettext;
-use gtk::subclass::prelude::*;
-use gtk::{gio, glib};
-use gtk::{prelude::*, CompositeTemplate};
 use std::os::unix::io::RawFd;
 
-use crate::CameraRow;
+use adw::subclass::prelude::*;
+use adw::traits::BreakpointBinExt;
+use ashpd::desktop::camera;
+use gettextrs::gettext;
+use gtk::{gio, glib};
+use gtk::{prelude::*, CompositeTemplate};
+
+use super::CameraControls;
 use crate::{config, utils};
 
 mod imp {
+    use std::cell::{Cell, OnceCell, RefCell};
+
     use super::*;
 
-    use gtk::glib::Properties;
-    use once_cell::unsync::OnceCell;
-    use std::cell::{Cell, RefCell};
-
-    #[derive(Debug, Default, CompositeTemplate, Properties)]
-    #[properties(wrapper_type = super::Camera)]
+    #[derive(Debug, Default, CompositeTemplate)]
     #[template(resource = "/org/gnome/Snapshot/ui/camera.ui")]
     pub struct Camera {
-        pub stream_list: RefCell<gio::ListStore>,
         pub selection: gtk::SingleSelection,
         pub provider: OnceCell<aperture::DeviceProvider>,
         pub players: RefCell<Option<gtk::MediaFile>>,
         settings: OnceCell<gio::Settings>,
 
-        #[property(get, set = Self::set_breakpoint, explicit_notify, builder(crate::Breakpoint::default()))]
-        pub breakpoint: Cell<crate::Breakpoint>,
+        pub recording_duration: Cell<u32>,
+        pub recording_source: RefCell<Option<glib::source::SourceId>>,
 
         #[template_child]
-        pub gallery_button: TemplateChild<crate::GalleryButton>,
+        pub single_landscape_bp: TemplateChild<adw::Breakpoint>,
         #[template_child]
-        pub camera_menu_button: TemplateChild<gtk::MenuButton>,
+        pub dual_landscape_bp: TemplateChild<adw::Breakpoint>,
         #[template_child]
-        pub camera_switch_button: TemplateChild<gtk::Button>,
+        pub dual_portrait_bp: TemplateChild<adw::Breakpoint>,
+
         #[template_child]
-        pub camera_menu_button_stack: TemplateChild<gtk::Stack>,
+        pub recording_revealer: TemplateChild<gtk::Revealer>,
+        #[template_child]
+        pub recording_label: TemplateChild<gtk::Label>,
+
         #[template_child]
         pub viewfinder: TemplateChild<aperture::Viewfinder>,
         #[template_child]
@@ -45,39 +47,20 @@ mod imp {
         pub stack: TemplateChild<gtk::Stack>,
         #[template_child]
         pub spinner: TemplateChild<gtk::Spinner>,
-        #[template_child]
-        pub shutter_button: TemplateChild<crate::ShutterButton>,
+
         #[template_child]
         pub guidelines: TemplateChild<crate::GuidelinesBin>,
 
         #[template_child]
-        pub camera_controls: TemplateChild<gtk::Box>,
-
+        pub camera_controls_vertical: TemplateChild<crate::CameraControls>,
         #[template_child]
-        pub sidebar_horizontal_start: TemplateChild<gtk::CenterBox>,
+        pub camera_controls_horizontal: TemplateChild<crate::CameraControls>,
+
         #[template_child]
         pub sidebar_horizontal_end: TemplateChild<gtk::CenterBox>,
         #[template_child]
-        pub sidebar_vertical_start: TemplateChild<gtk::CenterBox>,
-        #[template_child]
         pub sidebar_vertical_end: TemplateChild<gtk::CenterBox>,
 
-        #[template_child]
-        pub horizontal_start_countdown_button: TemplateChild<gtk::Widget>,
-        #[template_child]
-        pub horizontal_start_menu_button: TemplateChild<gtk::Widget>,
-        #[template_child]
-        pub horizontal_end_countdown_button: TemplateChild<gtk::Widget>,
-
-        #[template_child]
-        pub vertical_start_menu_button: TemplateChild<gtk::Widget>,
-        #[template_child]
-        pub vertical_end_menu_button: TemplateChild<gtk::Widget>,
-        #[template_child]
-        pub vertical_end_toggles: TemplateChild<gtk::Widget>,
-        #[template_child]
-        pub vertical_end_countdown_button: TemplateChild<gtk::Widget>,
-
         #[template_child]
         pub vertical_end_window_controls: TemplateChild<gtk::WindowControls>,
     }
@@ -86,12 +69,11 @@ mod imp {
     impl ObjectSubclass for Camera {
         const NAME: &'static str = "Camera";
         type Type = super::Camera;
-        type ParentType = gtk::Widget;
+        type ParentType = adw::BreakpointBin;
 
         fn class_init(klass: &mut Self::Class) {
             klass.bind_template();
             klass.bind_template_callbacks();
-            klass.set_layout_manager_type::<gtk::BinLayout>();
             klass.set_css_name("camera");
         }
 
@@ -107,131 +89,17 @@ mod imp {
                 .get_or_init(|| gio::Settings::new(config::APP_ID))
         }
 
-        fn set_breakpoint(&self, value: crate::Breakpoint) {
+        #[template_callback]
+        fn change_breakpoint(&self, breakpoint: adw::Breakpoint) {
             let obj = self.obj();
 
-            let is_vertical = matches!(
-                value,
-                crate::Breakpoint::SingleVertical | crate::Breakpoint::DualVertical
-            );
-            let is_mobile = matches!(
-                value,
-                crate::Breakpoint::DualVertical | crate::Breakpoint::DualHorizontal
-            );
-
-            self.sidebar_vertical_start.set_visible(is_vertical);
-            self.sidebar_vertical_end.set_visible(is_vertical);
-
-            self.sidebar_horizontal_start.set_visible(!is_vertical);
-            self.sidebar_horizontal_end.set_visible(!is_vertical);
-
-            self.sidebar_horizontal_end
-                .set_center_widget(gtk::Widget::NONE);
-
-            self.sidebar_vertical_end
-                .set_center_widget(gtk::Widget::NONE);
-
-            if is_vertical {
-                self.camera_controls
-                    .set_orientation(gtk::Orientation::Vertical);
-
-                self.camera_controls.set_margin_start(0);
-                self.camera_controls.set_margin_end(0);
-                self.camera_controls.set_margin_top(12);
-                self.camera_controls.set_margin_bottom(12);
-
-                self.sidebar_vertical_end
-                    .set_center_widget(Some(&self.camera_controls.get()));
-            } else {
-                self.camera_controls
-                    .set_orientation(gtk::Orientation::Horizontal);
-
-                self.camera_controls.set_margin_start(12);
-                self.camera_controls.set_margin_end(12);
-                self.camera_controls.set_margin_top(0);
-                self.camera_controls.set_margin_bottom(0);
-
-                self.sidebar_horizontal_end
-                    .set_center_widget(Some(&self.camera_controls.get()));
-            }
-
-            if is_mobile {
+            if breakpoint.eq(&self.dual_landscape_bp.get())
+                || breakpoint.eq(&self.dual_portrait_bp.get())
+            {
                 obj.add_css_class("mobile");
             } else {
                 obj.remove_css_class("mobile");
             }
-
-            match value {
-                crate::Breakpoint::SingleVertical => {
-                    if let Some(widget) = self.sidebar_vertical_start.center_widget() {
-                        widget.set_visible(false);
-                    }
-                    if let Some(widget) = self.sidebar_vertical_start.end_widget() {
-                        widget.set_visible(false);
-                    }
-
-                    self.vertical_start_menu_button.set_visible(false);
-                    self.vertical_end_menu_button.set_visible(true);
-                    self.vertical_end_toggles.set_visible(true);
-                    self.vertical_end_countdown_button.set_visible(true);
-                }
-                crate::Breakpoint::DualVertical => {
-                    if let Some(widget) = self.sidebar_vertical_start.center_widget() {
-                        widget.set_visible(true);
-                    }
-                    if let Some(widget) = self.sidebar_vertical_start.end_widget() {
-                        widget.set_visible(true);
-                    }
-
-                    self.vertical_start_menu_button.set_visible(true);
-                    self.vertical_end_menu_button.set_visible(false);
-                    self.vertical_end_toggles.set_visible(false);
-                    self.vertical_end_countdown_button.set_visible(false);
-                }
-                crate::Breakpoint::SingleHorizontal => {
-                    if let Some(widget) = self.sidebar_horizontal_start.center_widget() {
-                        widget.set_visible(false);
-                    }
-                    if let Some(widget) = self.sidebar_horizontal_end.end_widget() {
-                        widget.set_visible(true);
-                    }
-
-                    self.horizontal_start_countdown_button.set_visible(false);
-                    self.horizontal_start_menu_button.set_visible(false);
-                    self.horizontal_end_countdown_button.set_visible(true);
-                }
-                crate::Breakpoint::DualHorizontal => {
-                    if let Some(widget) = self.sidebar_horizontal_start.center_widget() {
-                        widget.set_visible(true);
-                    }
-                    if let Some(widget) = self.sidebar_horizontal_end.end_widget() {
-                        widget.set_visible(false);
-                    }
-
-                    self.horizontal_start_countdown_button.set_visible(true);
-                    self.horizontal_start_menu_button.set_visible(true);
-                    self.horizontal_end_countdown_button.set_visible(false);
-                }
-            }
-
-            if value != self.breakpoint.replace(value) {
-                obj.notify_breakpoint();
-            }
-        }
-
-        #[template_callback]
-        fn on_camera_switch_button_clicked(&self) {
-            let provider = self.provider.get().unwrap();
-
-            let current = self.viewfinder.camera();
-
-            let mut pos = 0;
-            if current == provider.camera(0) {
-                pos += 1;
-            };
-            if let Some(camera) = provider.camera(pos) {
-                self.viewfinder.set_camera(Some(camera));
-            }
         }
     }
 
@@ -241,9 +109,6 @@ mod imp {
 
             let obj = self.obj();
 
-            let popover = gtk::Popover::new();
-            popover.add_css_class("menu");
-
             let provider = aperture::DeviceProvider::instance();
             self.provider.set(provider.clone()).unwrap();
 
@@ -262,36 +127,8 @@ mod imp {
             obj.update_state();
 
             self.selection.set_model(Some(provider));
-            let factory = gtk::SignalListItemFactory::new();
-            factory.connect_setup(|_, item| {
-                let item = item.downcast_ref::<gtk::ListItem>().unwrap();
-                let camera_row = CameraRow::default();
-
-                item.set_child(Some(&camera_row));
-            });
-            let selection = &self.selection;
-            factory.connect_bind(glib::clone!(@weak selection => move |_, item| {
-                let item = item.downcast_ref::<gtk::ListItem>().unwrap();
-                let child = item.child().unwrap();
-                let row = child.downcast_ref::<CameraRow>().unwrap();
-
-                let item = item.item().and_downcast::<aperture::Camera>().unwrap();
-                row.set_item(&item);
-
-                selection.connect_selected_item_notify(glib::clone!(@weak row, @weak item => move |selection| {
-                    if let Some(selected_item) = selection.selected_item() {
-                        row.set_selected(selected_item == item);
-                    } else {
-                        row.set_selected(false);
-                    }
-                }));
-            }));
-            let list_view = gtk::ListView::new(Some(self.selection.clone()), Some(factory));
-
-            popover.set_child(Some(&list_view));
-
             self.selection.connect_selected_item_notify(
-                glib::clone!(@weak obj, @weak popover => move |selection| {
+                glib::clone!(@weak obj => move |selection| {
                     if let Some(selected_item) = selection.selected_item() {
                         let camera = selected_item.downcast::<aperture::Camera>().ok();
 
@@ -299,11 +136,24 @@ mod imp {
                             obj.imp().viewfinder.set_camera(camera);
                         }
                     }
-                    popover.popdown();
                 }),
             );
 
-            self.camera_menu_button.set_popover(Some(&popover));
+            self.camera_controls_horizontal
+                .set_selection(self.selection.clone());
+            self.camera_controls_vertical
+                .set_selection(self.selection.clone());
+
+            self.camera_controls_horizontal.connect_camera_switched(
+                glib::clone!(@weak self as obj => move |_: &CameraControls| {
+                    obj.obj().camera_switched();
+                }),
+            );
+            self.camera_controls_vertical.connect_camera_switched(
+                glib::clone!(@weak self as obj => move |_: &CameraControls| {
+                    obj.obj().camera_switched();
+                }),
+            );
 
             self.settings()
                 .bind(
@@ -323,33 +173,27 @@ mod imp {
                 }),
             );
 
-            // This makes sure the default state is properly set.
-            obj.set_breakpoint(crate::Breakpoint::default());
-        }
-
-        fn dispose(&self) {
-            self.dispose_template();
-        }
-
-        fn properties() -> &'static [glib::ParamSpec] {
-            Self::derived_properties()
-        }
-
-        fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-            Self::derived_property(self, id, pspec)
-        }
-
-        fn set_property(&self, id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
-            Self::derived_set_property(self, id, value, pspec)
+            obj.connect_current_breakpoint_notify(glib::clone!(@weak self as obj => move |imp| {
+                if imp.current_breakpoint().is_none()
+                || imp
+                    .current_breakpoint()
+                    .is_some_and(|breakpoint| breakpoint.eq(&obj.dual_portrait_bp.get()))
+            {
+                imp.add_css_class("portrait");
+            } else {
+                imp.remove_css_class("portrait");
+            }
+            }));
         }
     }
 
     impl WidgetImpl for Camera {}
+    impl BreakpointBinImpl for Camera {}
 }
 
 glib::wrapper! {
     pub struct Camera(ObjectSubclass<imp::Camera>)
-        @extends gtk::Widget;
+        @extends gtk::Widget, adw::BreakpointBin;
 }
 
 impl Default for Camera {
@@ -391,6 +235,7 @@ impl Camera {
         let path = utils::videos_dir()?.join(filename);
 
         self.imp().viewfinder.start_recording(path)?;
+        self.show_recording_label();
 
         Ok(())
     }
@@ -402,7 +247,8 @@ impl Camera {
         {
             if let Err(err) = imp.viewfinder.stop_recording() {
                 log::error!("Could not stop camera: {err}");
-            };
+            }
+            self.hide_recording_label();
         }
     }
 
@@ -427,6 +273,20 @@ impl Camera {
         Ok(())
     }
 
+    fn camera_switched(&self) {
+        let provider = self.imp().provider.get().unwrap();
+
+        let current = self.imp().viewfinder.camera();
+
+        let mut pos = 0;
+        if current == provider.camera(0) {
+            pos += 1;
+        };
+        if let Some(camera) = provider.camera(pos) {
+            self.imp().viewfinder.set_camera(Some(camera));
+        }
+    }
+
     fn play_shutter_sound(&self) {
         // If we don't hold a reference to it there is a condition race which
         // will cause the sound to play only sometimes.
@@ -437,27 +297,48 @@ impl Camera {
         self.imp().players.replace(Some(player));
     }
 
+    fn active_controls(&self) -> &CameraControls {
+        if self
+            .current_breakpoint()
+            .is_some_and(|bp| bp.to_string().contains("max-aspect-ratio: 4/3"))
+        {
+            &self.imp().camera_controls_horizontal
+        } else {
+            &self.imp().camera_controls_vertical
+        }
+    }
+
     pub fn set_countdown(&self, countdown: u32) {
-        self.imp().shutter_button.set_countdown(countdown);
+        self.imp()
+            .camera_controls_horizontal
+            .set_countdown(countdown);
+        self.imp().camera_controls_vertical.set_countdown(countdown);
     }
 
     pub fn start_countdown(&self) {
-        self.imp().shutter_button.start_countdown();
+        self.imp().camera_controls_horizontal.start_countdown();
+        self.imp().camera_controls_vertical.start_countdown();
     }
 
     pub fn stop_countdown(&self) {
-        self.imp().shutter_button.stop_countdown();
+        self.imp().camera_controls_horizontal.stop_countdown();
+        self.imp().camera_controls_vertical.stop_countdown();
     }
 
     pub fn shutter_mode(&self) -> crate::ShutterMode {
-        self.imp().shutter_button.shutter_mode()
+        self.active_controls().shutter_mode()
     }
 
     pub fn set_shutter_mode(&self, shutter_mode: crate::ShutterMode) {
         if matches!(shutter_mode, crate::ShutterMode::Picture) {
             self.stop_recording();
         }
-        self.imp().shutter_button.set_shutter_mode(shutter_mode);
+        self.imp()
+            .camera_controls_horizontal
+            .set_shutter_mode(shutter_mode);
+        self.imp()
+            .camera_controls_vertical
+            .set_shutter_mode(shutter_mode);
     }
 
     pub fn set_gallery(&self, gallery: crate::Gallery) {
@@ -480,12 +361,14 @@ impl Camera {
                 if let Some(file) = file {
                     gallery.add_video(file);
                 }
-                if matches!(imp.shutter_button.shutter_mode(), crate::ShutterMode::Recording) {
-                    imp.shutter_button.set_shutter_mode(crate::ShutterMode::Video);
+                if matches!(imp.camera_controls_horizontal.shutter_mode(), crate::ShutterMode::Recording) {
+                    imp.camera_controls_horizontal.set_shutter_mode(crate::ShutterMode::Video);
+                    imp.camera_controls_vertical.set_shutter_mode(crate::ShutterMode::Video);
                 }
             }),
         );
-        imp.gallery_button.set_gallery(&gallery);
+        imp.camera_controls_horizontal.set_gallery(&gallery);
+        imp.camera_controls_vertical.set_gallery(&gallery);
     }
 
     pub fn toggle_guidelines(&self) {
@@ -496,20 +379,12 @@ impl Camera {
     }
 
     fn update_cameras_button(&self, provider: &aperture::DeviceProvider) {
-        let imp = self.imp();
-        // NOTE We have a stack with an empty bin so that hiding the button does
-        // not ruin the layout.
-        match provider.n_items() {
-            0 | 1 => imp
-                .camera_menu_button_stack
-                .set_visible_child_name("fake-widget"),
-            2 => imp
-                .camera_menu_button_stack
-                .set_visible_child(&imp.camera_switch_button.get()),
-            _ => imp
-                .camera_menu_button_stack
-                .set_visible_child(&imp.camera_menu_button.get()),
-        }
+        self.imp()
+            .camera_controls_horizontal
+            .update_visible_camera_button(provider.n_items());
+        self.imp()
+            .camera_controls_vertical
+            .update_visible_camera_button(provider.n_items());
     }
 
     fn update_state(&self) {
@@ -548,6 +423,44 @@ impl Camera {
         imp.vertical_end_window_controls
             .set_decoration_layout(decoration_layout.as_deref());
     }
+
+    fn show_recording_label(&self) {
+        let imp = self.imp();
+
+        let source = glib::timeout_add_seconds_local(
+            1,
+            glib::clone!(@weak self as obj => @default-return glib::ControlFlow::Break, move || {
+                let imp = obj.imp();
+
+                // TODO Use Cell::update once stabilized.
+                let duration = imp.recording_duration.get() + 1;
+                imp.recording_duration.set(duration);
+
+                let minutes = duration.div_euclid(60);
+                let seconds = duration.rem_euclid(60);
+                imp.recording_label.set_label(&format!("{minutes}∶{seconds:02}"));
+
+                glib::ControlFlow::Continue
+            }),
+        );
+        if let Some(old) = imp.recording_source.replace(Some(source)) {
+            old.remove();
+        }
+        imp.recording_duration.set(0);
+        imp.recording_revealer.set_reveal_child(true);
+        imp.recording_label.set_label("0∶00");
+    }
+
+    fn hide_recording_label(&self) {
+        let imp = self.imp();
+
+        if let Some(source) = imp.recording_source.take() {
+            source.remove();
+            imp.recording_duration.set(0);
+            imp.recording_label.set_label("0∶00");
+            imp.recording_revealer.set_reveal_child(false);
+        }
+    }
 }
 
 async fn stream() -> ashpd::Result<RawFd> {
